package com.ghomerr.linkedchest.listeners;

import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.block.Block;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.inventory.Inventory;

import com.ghomerr.linkedchest.LinkedChest;
import com.ghomerr.linkedchest.enums.Commands;
import com.ghomerr.linkedchest.enums.Messages;
import com.ghomerr.linkedchest.utils.DebugUtils;
import com.ghomerr.linkedchest.utils.MessagesUtils;
import com.ghomerr.linkedchest.utils.PermissionsUtils;
import com.ghomerr.linkedchest.utils.StringUtils;
import com.ghomerr.linkedchest.utils.WorldUtils;

public class CommandsListener implements CommandExecutor
{
	public LinkedChest plugin = null;

	public CommandsListener (final LinkedChest plugin)
	{
		this.plugin = plugin;
	}

	@Override
	public boolean onCommand(final CommandSender sender, final Command cmd, final String label, final String[] split)
	{
		if (sender instanceof Player)
		{
			final Player player = (Player) sender;

			final Commands main = Commands.commands.get(label);
			switch (main)
			{
				case LINKED_CHEST:
				{
					if (split.length >= 1)
					{
						final String arg1 = split[0];
						final Commands command = Commands.commands.get(arg1);

						if (command != null)
						{
							if (PermissionsUtils.hasPermission(player, command))
							{
								switch (command)
								{
									case ADD:
									{
										if (split.length == 2 || split.length == 3)
										{
											final String chestName = split[1];
		
											final Inventory virtualInv = plugin.getVirtualInventory(chestName);
		
											if (virtualInv == null)
											{
												final Block block = player.getTargetBlock(null, 6);
		
												if (WorldUtils.isChest(block))
												{
													String checkChestName = plugin.getMasterChestNameFromBlock(block);
													
													if (checkChestName != null)
													{
														player.sendMessage(MessagesUtils.getWithColors(Messages.TARGET_CHEST_ALREADY_MASTER,
																ChatColor.RED, ChatColor.AQUA, chestName, checkChestName));
													}
													else
													{
														boolean isAdminChest = false;
														String adminOption = null;
			
														if (split.length == 3)
														{
															adminOption = split[2];
															isAdminChest = Commands.ADMIN_CHEST.has(adminOption);
														}
			
														if (plugin.addVirtualInventory(chestName, block, isAdminChest))
														{
			
															player.sendMessage(MessagesUtils.getWithColors(
																	Messages.MASTER_CHEST_ADDED, ChatColor.GREEN, ChatColor.AQUA,
																	chestName));
															
															final Location nearbyChestLoc = WorldUtils.getChestNearby(block, Location.class);
															if (nearbyChestLoc != null)
															{
																plugin.addDoubleChestLocation(nearbyChestLoc, chestName);
															}
														}
														else
														{
															player.sendMessage(MessagesUtils.getWithColors(
																	Messages.INVALID_CHEST_NAME, ChatColor.RED, ChatColor.AQUA,
																	chestName));
														}
													}
												}
											}
											else
											{
												player.sendMessage(MessagesUtils.getWithColors(Messages.CHEST_ALREADY_EXISTS,
														ChatColor.RED, ChatColor.AQUA, chestName));
											}
										}
										else
										{
											player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_ADD, ChatColor.YELLOW));
										}
										break;
									}
		
									case ADMIN_CHEST:
									{
										if (split.length == 2)
										{
											final String chestName = split[1];
											boolean isAdmin = false;
											if (plugin.hasChestName(chestName))
											{
												try
												{
													isAdmin = plugin.toggleAdminChest(chestName);
													if (isAdmin)
													{
														player.sendMessage(MessagesUtils.getWithColors(Messages.CHEST_IS_ADMIN,
																ChatColor.RED, ChatColor.AQUA, chestName));
													}
													else
													{
														player.sendMessage(MessagesUtils.getWithColors(Messages.CHEST_ISNOT_ADMIN,
																ChatColor.GREEN, ChatColor.AQUA, chestName));
													}
												}
												catch (final IllegalArgumentException ex)
												{
													player.sendMessage(MessagesUtils.getWithColors(Messages.CHEST_ADMIN_FAILED,
															ChatColor.RED, ChatColor.AQUA, chestName));
												}
											}
											else
											{
												player.sendMessage(MessagesUtils.getWithColors(Messages.CHEST_NOT_FOUND,
														ChatColor.RED, ChatColor.AQUA, chestName));
											}
										}
										else
										{
											player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_ALIASES, ChatColor.YELLOW));
										}
										break;
									}
		
									case ALIASES:
									{
										if (split.length == 2)
										{
											final String helpCommand = split[1];
											if (StringUtils.isNotBlank(helpCommand))
											{
												final Commands helpCmd = Commands.commands.get(helpCommand);
												if (helpCmd != null)
												{
													player.sendMessage(plugin.getAliasesListForCommand(helpCmd));
												}
												else
												{
													player.sendMessage(MessagesUtils.getWithColors(
															Messages.ALIASES_UNKNOWN_COMMAND, ChatColor.RED, ChatColor.AQUA,
															helpCommand));
												}
											}
										}
										else
										{
											player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_ALIASES,
													ChatColor.YELLOW));
										}
										break;
									}
		
									case CONFIG:
									{
										player.sendMessage(plugin.getConfigList());
										break;
									}
		
									case DEBUG:
									{
										final boolean isDebug = DebugUtils.toggleDebug();
										if (isDebug)
										{
											player.sendMessage(MessagesUtils.getWithColor(Messages.DEBUG_ENABLED,
													ChatColor.GREEN));
										}
										else
										{
											player.sendMessage(MessagesUtils.getWithColor(Messages.DEBUG_DISABLED,
													ChatColor.RED));
										}
										break;
									}
									
									case DETAILS:
									{
										if (split.length == 2)
										{
											final String chestName = split[1];
											player.sendMessage(plugin.getChestDetails(chestName));
										}
										else
										{
											player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_DETAILS,
													ChatColor.YELLOW));
										}
										break;
									}
		
									case LIST:
									{
										player.sendMessage(plugin.getLinkedChestsList());
										break;
									}
		
									case OPEN:
									{
										if (split.length == 2)
										{
											final String chestName = split[1];
											if (plugin.hasChestName(chestName))
											{
												if (plugin.openVirtualInventory(player, chestName))
												{
													player.sendMessage(MessagesUtils.getWithColors(Messages.REMOTE_OPENING_CHEST,
															ChatColor.YELLOW, ChatColor.AQUA, chestName));
												}
												else
												{
													player.sendMessage(MessagesUtils.getWithColors(Messages.OPEN_CHEST_FAILED,
															ChatColor.RED, ChatColor.AQUA, chestName));
												}
											}
											else
											{
												player.sendMessage(MessagesUtils.getWithColors(Messages.CHEST_NOT_FOUND,
														ChatColor.RED, ChatColor.AQUA, chestName));
											}
										}
										else
										{
											player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_OPEN, ChatColor.YELLOW));
										}
										break;
									}
		
									case REMOVE:
									{
										if (split.length == 2)
										{
											final String chestName = split[1];
											if (plugin.hasChestName(chestName))
											{
												if (plugin.removeVirtualInventory(chestName))
												{
													player.sendMessage(MessagesUtils.getWithColors(Messages.CHEST_REMOVED,
															ChatColor.GREEN, ChatColor.AQUA, chestName));
												}
												else
												{
													player.sendMessage(MessagesUtils.getWithColors(Messages.REMOVE_CHEST_FAILED,
															ChatColor.RED, ChatColor.AQUA, chestName));
												}
											}
											else
											{
												player.sendMessage(MessagesUtils.getWithColors(Messages.CHEST_NOT_FOUND,
														ChatColor.RED, ChatColor.AQUA, chestName));
											}
										}
										else
										{
											player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_REMOVE,
													ChatColor.YELLOW));
										}
										break;
									}
		
									case USE_PERMISSIONS:
									{
										final boolean isPermsEnabled = PermissionsUtils.togglePermissionsState();
										if (isPermsEnabled)
										{
											player.sendMessage(MessagesUtils.getWithColor(Messages.PERMISSIONS_ENABLED,
													ChatColor.GREEN));
										}
										else
										{
											player.sendMessage(MessagesUtils.getWithColor(Messages.PERMISSIONS_DISABLED,
													ChatColor.RED));
										}
										break;
									}
		
									case HELP:
									{
										String helpCommand = null;
		
										if (split.length == 2)
										{
											helpCommand = split[1];
											if (StringUtils.isNotBlank(helpCommand))
											{
												final Commands helpCmd = Commands.commands.get(helpCommand);
												switch (helpCmd)
												{
													case ADD:
														player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_ADD,
																ChatColor.YELLOW));
														break;
													case ADMIN_CHEST:
														player.sendMessage(MessagesUtils.getWithColor(
																Messages.HELP_ADMIN_CHEST, ChatColor.YELLOW));
														break;
													case ALIASES:
														player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_ALIASES,
																ChatColor.YELLOW));
														break;
													case CONFIG:
														player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_CONFIG,
																ChatColor.YELLOW));
														break;
													case DEBUG:
														player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_DEBUG,
																ChatColor.YELLOW));
													case DETAILS:
														player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_DETAILS,
																ChatColor.YELLOW));
														break;
													case HELP:
														player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_HELP,
																ChatColor.YELLOW));
														break;
													case LINKED_CHEST:
														player.sendMessage(MessagesUtils.getWithColor(
																Messages.HELP_LINKED_CHEST, ChatColor.YELLOW));
														break;
													case LIST:
														player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_LIST,
																ChatColor.YELLOW));
														break;
													case OPEN:
														player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_OPEN,
																ChatColor.YELLOW));
														break;
													case REMOVE:
														player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_REMOVE,
																ChatColor.YELLOW));
														break;
													case USE_PERMISSIONS:
														player.sendMessage(MessagesUtils.getWithColor(
																Messages.HELP_USE_PERMISSIONS, ChatColor.YELLOW));
														break;
													default:
														player.sendMessage(MessagesUtils.getWithColor(
																Messages.HELP_INVALID_CMD, ChatColor.RED));
												}
											}
											else
											{
												player.sendMessage(MessagesUtils.getWithColor(Messages.HELP_INVALID_CMD,
														ChatColor.RED));
											}
										}
										else
										{
											plugin.displayCompleteHelpMessage(player);
										}
										break;
									}
								}
							}
						}
						else
						{
							player.sendMessage(MessagesUtils.getWithColors(Messages.UNKNOWN_COMMAND, ChatColor.RED,
									ChatColor.AQUA, arg1));
							plugin.displayCompleteHelpMessage(player);
						}
					}
				}
			}
		}
		return true;
	}
}
