package com.ghomerr.linkedchest.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Logger;

import com.ghomerr.linkedchest.constants.Constants;
import com.ghomerr.linkedchest.objects.SaveThread;

public class FileUtils
{
	private static final Logger _LOGGER = Logger.getLogger(Constants.MINECRAFT);
	public static boolean isDebugEnabled = false;
	
	public static void saveFile(final Properties data, final String fileName)
	{
		final Thread th = new Thread(new SaveThread(data, fileName));
		th.run();
	}
	
	public static Properties loadData(final String fileName)
	{
		final File pluginDir = new File(Constants.PLUGIN_DIR_PATH);		
		if (!pluginDir.exists())
		{
			try
			{
				pluginDir.mkdir();
				pluginDir.setExecutable(true, false);
				pluginDir.setReadable(true, false);
				pluginDir.setWritable(true, false);
			}
			catch (final Throwable th)
			{
				_LOGGER.severe(Constants.TAG + "Unable to create plugin dir " + Constants.PLUGIN_DIR_PATH);
				th.printStackTrace();
				return null;
			}
		}
		
		final Properties data = new Properties();
		File dataFile = new File(fileName);
		
		if (!dataFile.exists())
		{
			dataFile = createNewFile(fileName);
			if (dataFile == null || !dataFile.exists())
			{
				_LOGGER.severe(Constants.TAG + "Unable to find or create file " + fileName);
				return null;
			}
		}		
		
		FileInputStream in = null;
		try
		{
			in = new FileInputStream(dataFile);
		}
		catch (final Throwable ex)
		{
			_LOGGER.severe(Constants.TAG + "Unable to create a stream to read the file " + fileName);
			ex.printStackTrace();
			return null;
		}
	
		try
		{
			data.load(in);
			in.close();
			return data;
		}
		catch (final IOException ex)
		{
			_LOGGER.severe(Constants.TAG + "Error while loading the file " + fileName);
			ex.printStackTrace();
			return null;
		}
	}
	
	public static File createNewFile(final String fileName)
	{
		final File file = new File(fileName);
		
		if (!file.exists())
		{
			try
			{
				file.createNewFile();
				file.setExecutable(true, false);
				file.setReadable(true, false);
				file.setWritable(true, false);
				return file;
			}
			catch (final Throwable th)
			{
				_LOGGER.severe(Constants.TAG + "Error while creating file " + fileName);
				th.printStackTrace();
				return null;
			}
		}
		else
		{
			return file;
		}
	}
}
